@@ -1,26 +1,33 @@
 # encoding=utf-8
 '''Document content post-processing.'''
 import abc
+import codecs
 import gettext
 import io
 import logging
 import os.path
-import re
 import shutil
 
-import lxml.html
-
 from wpull.backport.logging import BraceMessage as __
-from wpull.database import Status
-from wpull.scraper import HTMLScraper, CSSScraper
-import wpull.string
+from wpull.database.base import NotFound
+from wpull.document.htmlparse.element import Comment, Element, Doctype
+from wpull.item import Status
+from wpull.scraper.css import CSSScraper
+from wpull.scraper.html import HTMLScraper, ElementWalker
 from wpull.url import URLInfo
+import wpull.string
 
 
 _ = gettext.gettext
 _logger = logging.getLogger(__name__)
 
 
+# Snipped from lxml.html.def:
+empty_tags = frozenset([
+    'area', 'base', 'basefont', 'br', 'col', 'frame', 'hr',
+    'img', 'input', 'isindex', 'link', 'meta', 'param'])
+
+
 class BaseDocumentConverter(object, metaclass=abc.ABCMeta):
     '''Base class for classes that convert links within a document.'''
     @abc.abstractmethod
@@ -35,15 +42,16 @@ class BatchDocumentConverter(object):
         url_table: An instance of :class:`.database.URLTable`.
         backup (bool): Whether back up files are created.
     '''
-    def __init__(self, url_table, backup=False):
+    def __init__(self, html_parser, element_walker, url_table, backup=False):
         self._url_table = url_table
         self._backup_enabled = backup
-        self._html_converter = HTMLConverter(url_table)
+        self._html_converter = HTMLConverter(html_parser, element_walker,
+                                             url_table)
         self._css_converter = CSSConverter(url_table)
 
     def convert_all(self):
         '''Convert all links in URL table.'''
-        for url_record in self._url_table.values():
+        for url_record in self._url_table.get_all():
             if url_record.status != Status.done:
                 continue
 
@@ -97,8 +105,8 @@ class BatchDocumentConverter(object):
 
 class HTMLConverter(HTMLScraper, BaseDocumentConverter):
     '''HTML converter.'''
-    def __init__(self, url_table):
-        super().__init__()
+    def __init__(self, html_parser, element_walker, url_table):
+        super().__init__(html_parser, element_walker)
         self._url_table = url_table
         self._css_converter = CSSConverter(url_table)
         self._out_file = None
@@ -116,12 +124,18 @@ class HTMLConverter(HTMLScraper, BaseDocumentConverter):
             )
 
         with open(input_filename, 'rb') as in_file:
-            doctype = self.parse_doctype(in_file, encoding=encoding)
-            is_xhtml = doctype and 'XHTML' in doctype
+            try:
+                doctype = self._html_parser.parse_doctype(in_file,
+                                                          encoding=encoding)
+                is_xhtml = doctype and 'XHTML' in doctype
+            except AttributeError:
+                # using html5lib
+                is_xhtml = False
+                doctype = None
 
         with open(input_filename, 'rb') as in_file:
             with open(output_filename, 'wb') as bin_out_file:
-                elements = self.read_tree(in_file, encoding=encoding)
+                elements = self.iter_elements(in_file, encoding=encoding)
                 out_file = io.TextIOWrapper(bin_out_file, encoding=encoding)
 
                 if doctype:
@@ -132,18 +146,23 @@ class HTMLConverter(HTMLScraper, BaseDocumentConverter):
                 self._encoding = encoding
 
                 for element in elements:
-                    if element.tag == wpull.document.COMMENT:
+                    if isinstance(element, Comment):
                         out_file.write(
                             '<!--{0}-->'.format(element.text)
                         )
-                    elif element.end:
-                        if element.tag not in lxml.html.defs.empty_tags:
-                            self._out_file.write('</{0}>'.format(element.tag))
+                    elif isinstance(element, Element):
+                        if element.end:
+                            if element.tag not in empty_tags:
+                                self._out_file.write('</{0}>'
+                                                     .format(element.tag))
 
-                        if element.tail:
-                            self._out_file.write(element.tail)
-                    else:
-                        self._convert_element(element, is_xhtml=is_xhtml)
+                            if element.tail:
+                                self._out_file.write(element.tail)
+                        else:
+                            self._convert_element(element, is_xhtml=is_xhtml)
+                    elif isinstance(element, Doctype):
+                        doctype = element.text
+                        is_xhtml = doctype and 'XHTML' in doctype
 
                 self._out_file.close()
                 self._out_file = None
@@ -156,7 +175,7 @@ class HTMLConverter(HTMLScraper, BaseDocumentConverter):
         unfilled_value = object()
         new_attribs = dict(((name, unfilled_value) for name in element.attrib))
 
-        for link_info in self.iter_links_element(element):
+        for link_info in self._element_walker.iter_links_element(element):
             new_value = None
 
             if link_info.value_type == 'plain':
@@ -170,7 +189,7 @@ class HTMLConverter(HTMLScraper, BaseDocumentConverter):
                     if text:
                         new_text = text
 
-            if new_value:
+            if new_value and link_info.attrib:
                 if new_attribs[link_info.attrib] == unfilled_value:
                     new_attribs[link_info.attrib] = [new_value]
                 else:
@@ -184,12 +203,12 @@ class HTMLConverter(HTMLScraper, BaseDocumentConverter):
 
             self._out_file.write(' {0}="{1}"'.format(name, value))
 
-        if is_xhtml and element.tag in lxml.html.defs.empty_tags:
+        if is_xhtml and element.tag in empty_tags:
             self._out_file.write('/')
 
         self._out_file.write('>')
 
-        if element.tag not in lxml.html.defs.empty_tags:
+        if element.tag not in empty_tags:
             if new_text:
                 self._out_file.write(new_text)
 
@@ -240,12 +259,15 @@ class HTMLConverter(HTMLScraper, BaseDocumentConverter):
             text, base_url=self._base_url
         )
 
-        self._css_already_done.add(link_info.element)
+        self._css_already_done.add(id(link_info.element))
 
         return new_text
 
     def _get_new_url(self, url_info):
-        url_record = self._url_table.get(url_info.url)
+        try:
+            url_record = self._url_table.get_one(url_info.url)
+        except NotFound:
+            url_record = None
 
         if url_record \
            and url_record.status == Status.done and url_record.filename:
@@ -258,39 +280,46 @@ class HTMLConverter(HTMLScraper, BaseDocumentConverter):
 
 class CSSConverter(CSSScraper, BaseDocumentConverter):
     '''CSS converter.'''
-    ALL_URL_PATTERN = r'{0}|{1}'.format(
-        CSSScraper.URL_PATTERN, CSSScraper.IMPORT_URL_PATTERN)
-
     def __init__(self, url_table):
         super().__init__()
         self._url_table = url_table
 
     def convert(self, input_filename, output_filename, base_url=None):
-        with open(input_filename, 'rb') as in_file:
-            text = in_file.read()
+        with open(input_filename, 'rb') as in_file, \
+                open(output_filename, 'wb') as out_file:
+            encoding = wpull.string.detect_encoding(
+                wpull.util.peek_file(in_file))
+            out_stream = codecs.getwriter(encoding)(out_file)
 
-        encoding = wpull.string.detect_encoding(text)
-        text = text.decode(encoding)
-        new_text = self.convert_text(text, base_url)
-
-        with open(output_filename, 'wb') as out_file:
-            out_file.write(new_text.encode(encoding))
+            for text, is_link in self.iter_processed_text(in_file, encoding):
+                if is_link:
+                    out_stream.write(self.get_new_url(text, base_url))
+                else:
+                    out_stream.write(text)
 
     def convert_text(self, text, base_url=None):
-        def repl(match):
-            url = match.group(1) or match.group(2)
-
-            if base_url:
-                url = wpull.url.urljoin(base_url, url)
-
-            url_record = self._url_table.get(url)
-
-            if url_record \
-               and url_record.status == Status.done and url_record.filename:
-                new_url = url_record.filename
+        text_list = []
+        for text, is_link in self.iter_processed_text(io.StringIO(text)):
+            if is_link:
+                text_list.append(self.get_new_url(text, base_url))
             else:
-                new_url = url
+                text_list.append(text)
 
-            return match.group().replace(url, new_url)
+        return ''.join(text_list)
 
-        return re.sub(self.ALL_URL_PATTERN, repl, text)
+    def get_new_url(self, url, base_url=None):
+        if base_url:
+            url = wpull.url.urljoin(base_url, url)
+
+        url_record = self._url_table.get_one(url)
+
+        if url_record \
+           and url_record.status == Status.done and url_record.filename:
+            new_url = url_record.filename
+        else:
+            new_url = url
+
+        return new_url
+
+
+# TODO: add javascript conversion
