@@ -28,7 +28,7 @@ class TestURL(unittest.TestCase):
     def test_url_info_naked(self):
         self.assertEqual(
             'http://example.com/',
-            URLInfo.parse('example.com').url
+            URLInfo.parse('Example.Com').url
         )
         self.assertEqual(
             'http://example.com/',
@@ -68,26 +68,48 @@ class TestURL(unittest.TestCase):
 
     def test_url_info_parts(self):
         url_info = URLInfo.parse(
-            'HTTP://userName:pass%3Aword@[::1]:81/ásdF/ghjK?a=b=c&D#/?')
+            'HTTP://userName:pass%3Aword@[A::1]:81/ásdF/ghjK?a=b=c&D#/?')
         self.assertEqual(
-            'http://userName:pass:word@[::1]:81/%C3%A1sdF/ghjK?a=b=c&D',
+            'http://userName:pass:word@[a::1]:81/%C3%A1sdF/ghjK?a=b=c&D',
             url_info.url
         )
         self.assertEqual('http', url_info.scheme)
-        self.assertEqual('userName:pass%3Aword@[::1]:81',
+        self.assertEqual('userName:pass%3Aword@[A::1]:81',
                          url_info.authority)
         self.assertEqual('/ásdF/ghjK?a=b=c&D#/?', url_info.resource)
         self.assertEqual('userName', url_info.username)
         self.assertEqual('pass:word', url_info.password)
-        self.assertEqual('[::1]:81', url_info.host)
-        self.assertEqual('::1', url_info.hostname)
+        self.assertEqual('[A::1]:81', url_info.host)
+        self.assertEqual('[a::1]:81', url_info.hostname_with_port)
+        self.assertEqual('a::1', url_info.hostname)
         self.assertEqual(81, url_info.port)
         self.assertEqual('/%C3%A1sdF/ghjK', url_info.path)
         self.assertEqual('a=b=c&D', url_info.query)
         self.assertEqual('/?', url_info.fragment)
         self.assertEqual('utf-8', url_info.encoding)
         self.assertEqual(
-            'HTTP://userName:pass%3Aword@[::1]:81/ásdF/ghjK?a=b=c&D#/?',
+            'HTTP://userName:pass%3Aword@[A::1]:81/ásdF/ghjK?a=b=c&D#/?',
+            url_info.raw)
+
+        url_info = URLInfo.parse(
+            'Ftp://N00B:hunter2@LocalHost.Example/mydocs/'
+        )
+        self.assertEqual('ftp', url_info.scheme)
+        self.assertEqual('N00B:hunter2@LocalHost.Example',
+                         url_info.authority)
+        self.assertEqual('/mydocs/', url_info.resource)
+        self.assertEqual('N00B', url_info.username)
+        self.assertEqual('hunter2', url_info.password)
+        self.assertEqual('LocalHost.Example', url_info.host)
+        self.assertEqual('localhost.example', url_info.hostname_with_port)
+        self.assertEqual('localhost.example', url_info.hostname)
+        self.assertEqual(21, url_info.port)
+        self.assertEqual('/mydocs/', url_info.path)
+        self.assertFalse(url_info.query)
+        self.assertFalse(url_info.fragment)
+        self.assertEqual('utf-8', url_info.encoding)
+        self.assertEqual(
+            'Ftp://N00B:hunter2@LocalHost.Example/mydocs/',
             url_info.raw)
 
     def test_url_info_default_port(self):
