@@ -11,12 +11,15 @@ import http.client
 import itertools
 import logging
 import os
+import re
 import shutil
 import time
 import urllib.parse
 
 from wpull.backport.logging import BraceMessage as __
 from wpull.body import Body
+from wpull.document.css import CSSReader
+from wpull.document.html import HTMLReader
 import wpull.util
 
 
@@ -95,13 +98,24 @@ class BaseFileWriter(BaseWriter):
             responses on top of the document
         local_timestamping: If True, the writer will set the Last-Modified
             timestamp on downloaded files
+        adjust_extension: If True, HTML or CSS file extension will be added
+            whenever it is detected as so.
+        content_disposition: If True, the filename is extracted from
+            the Content-Disposition header.
+        trust_server_names: If True and there is redirection, use the last
+            given response for the filename.
     '''
     def __init__(self, path_namer, file_continuing=False,
-                 headers_included=False, local_timestamping=True):
+                 headers_included=False, local_timestamping=True,
+                 adjust_extension=False, content_disposition=False,
+                 trust_server_names=False):
         self._path_namer = path_namer
         self._file_continuing = file_continuing
         self._headers_included = headers_included
         self._local_timestamping = local_timestamping
+        self._adjust_extension = adjust_extension
+        self._content_disposition = content_disposition
+        self._trust_server_names = trust_server_names
 
     @abc.abstractproperty
     def session_class(self):
@@ -118,17 +132,25 @@ class BaseFileWriter(BaseWriter):
             self._file_continuing,
             self._headers_included,
             self._local_timestamping,
+            self._adjust_extension,
+            self._content_disposition,
+            self._trust_server_names,
         )
 
 
 class BaseFileWriterSession(BaseWriterSession):
     '''Base class for File Writer Sessions.'''
     def __init__(self, path_namer, file_continuing,
-                 headers_included, local_timestamping):
+                 headers_included, local_timestamping,
+                 adjust_extension, content_disposition,
+                 trust_server_names):
         self._path_namer = path_namer
         self._file_continuing = file_continuing
         self._headers_included = headers_included
         self._local_timestamping = local_timestamping
+        self._adjust_extension = adjust_extension
+        self._content_disposition = content_disposition
+        self._trust_server_names = trust_server_names
         self._filename = None
 
     @classmethod
@@ -237,6 +259,15 @@ class BaseFileWriterSession(BaseWriterSession):
             if self._file_continuing:
                 self._process_file_continue_response(response)
             elif 200 <= code <= 299 or 400 <= code:
+                if self._trust_server_names:
+                    self._rename_with_last_response(response)
+
+                if self._content_disposition:
+                    self._rename_with_content_disposition(response)
+
+                if self._adjust_extension:
+                    self._append_filename_extension(response)
+
                 self.open_file(self._filename, response)
 
     def _process_file_continue_response(self, response):
@@ -250,6 +281,55 @@ class BaseFileWriterSession(BaseWriterSession):
                 _('Could not continue file download: {filename}.')
                 .format(filename=self._filename))
 
+    def _append_filename_extension(self, response):
+        '''Append an HTML/CSS file suffix as needed.'''
+        if not self._filename:
+            return
+
+        if response.request.url_info.scheme not in ('http', 'https'):
+            return
+
+        if not re.search(r'\.[hH][tT][mM][lL]?$', self._filename) and \
+                HTMLReader.is_response(response):
+            self._filename += '.html'
+        elif not re.search(r'\.[cC][sS][sS]$', self._filename) and \
+                CSSReader.is_response(response):
+            self._filename += '.css'
+
+    def _rename_with_content_disposition(self, response):
+        '''Rename using the Content-Disposition header.'''
+        if not self._filename:
+            return
+
+        if response.request.url_info.scheme not in ('http', 'https'):
+            return
+
+        header_value = response.fields.get('Content-Disposition')
+
+        if not header_value:
+            return
+
+        filename = parse_content_disposition(header_value)
+
+        if filename:
+            parts = list(self._filename.split(os.sep))
+
+            if parts:
+                del parts[-1]
+
+            new_filename = self._path_namer.safe_filename(filename)
+            parts.append(new_filename)
+            self._filename = os.path.join(*parts)
+
+    def _rename_with_last_response(self, response):
+        if not self._filename:
+            return
+
+        if response.request.url_info.scheme not in ('http', 'https'):
+            return
+
+        self._filename = self._compute_filename(response.request)
+
     def save_document(self, response):
         if self._filename and os.path.exists(self._filename):
             if self._headers_included:
@@ -437,18 +517,19 @@ class PathNamer(BasePathNamer):
             alt_char=alt_char
         ))
 
-        parts = [
-            safe_filename(
-                part,
-                os_type=self._os_type, no_control=self._no_control,
-                ascii_only=self._ascii_only, case=self._case,
-                max_length=self._max_filename_length,
-            )
-            for part in parts
-        ]
+        parts = [self.safe_filename(part) for part in parts]
 
         return os.path.join(self._root, *parts)
 
+    def safe_filename(self, part):
+        '''Return a safe filename or file part.'''
+        return safe_filename(
+            part,
+            os_type=self._os_type, no_control=self._no_control,
+            ascii_only=self._ascii_only, case=self._case,
+            max_length=self._max_filename_length,
+        )
+
 
 def url_to_filename(url, index='index.html', alt_char=False):
     '''Return a filename from a URL.
@@ -664,3 +745,25 @@ def anti_clobber_dir_path(dir_path, suffix='.d'):
             return os.path.join(*parts)
 
     return dir_path
+
+
+def parse_content_disposition(text):
+    '''Parse a Content-Disposition header value.'''
+    match = re.search(r'filename\s*=\s*(.+)', text, re.IGNORECASE)
+
+    if not match:
+        return
+
+    filename = match.group(1)
+
+    if filename[0] in '"\'':
+        match = re.match(r'(.)(.+)(?!\\)\1', filename)
+
+        if match:
+            filename = match.group(2).replace('\\"', '"')
+
+            return filename
+
+    else:
+        filename = filename.partition(';')[0].strip()
+        return filename
