@@ -11,7 +11,7 @@ from wpull.options import AppArgumentParser
 import wpull.testing.async
 from wpull.testing.goodapp import GoodAppTestCase
 from wpull.writer import (url_to_dir_parts, url_to_filename, safe_filename,
-                          anti_clobber_dir_path)
+                          anti_clobber_dir_path, parse_content_disposition)
 
 
 DEFAULT_TIMEOUT = 30
@@ -161,6 +161,70 @@ class TestWriter(unittest.TestCase):
                 anti_clobber_dir_path('a/b/c/d/e/f/g')
             )
 
+    def test_parse_content_disposition(self):
+        self.assertEqual(
+            'hello.txt',
+            parse_content_disposition('attachment; filename=hello.txt')
+        )
+        self.assertEqual(
+            'hello.txt',
+            parse_content_disposition(
+                'attachment; filename=hello.txt; filename*=blahblah')
+        )
+        self.assertEqual(
+            'hello.txt',
+            parse_content_disposition(
+                'attachment; filename=hello.txt ;filename*=blahblah')
+        )
+        self.assertEqual(
+            'hello.txt',
+            parse_content_disposition('attachment; filename="hello.txt"')
+        )
+        self.assertEqual(
+            'hello.txt',
+            parse_content_disposition('attachment; filename="hello.txt" ;')
+        )
+        self.assertEqual(
+            'hello world',
+            parse_content_disposition('attachment; filename="hello world"')
+        )
+        self.assertEqual(
+            'hello world',
+            parse_content_disposition('attachment; filename="hello world"')
+        )
+        self.assertEqual(
+            'hello world',
+            parse_content_disposition("attachment; filename='hello world'")
+        )
+        self.assertEqual(
+            'hello"world',
+            parse_content_disposition('attachment; filename="hello\\"world"')
+        )
+        self.assertEqual(
+            '\'hello"world\'',
+            parse_content_disposition('attachment; filename="\'hello\\"world\'"')
+        )
+        self.assertEqual(
+            '\'hello"world\'',
+            parse_content_disposition(
+                'attachment; filename="\'hello\\"world\'";')
+        )
+        self.assertFalse(
+            parse_content_disposition('attachment; filename=')
+        )
+        self.assertFalse(
+            parse_content_disposition('attachment; filename=""')
+        )
+        self.assertFalse(
+            parse_content_disposition('attachment; filename=";')
+        )
+        self.assertFalse(
+            parse_content_disposition('attachment; filename=\'aaa')
+        )
+        self.assertFalse(
+            parse_content_disposition('attachment; filename="aaa')
+        )
+
 
 class TestWriterApp(GoodAppTestCase):
     @wpull.testing.async.async_test(timeout=DEFAULT_TIMEOUT)
@@ -336,3 +400,83 @@ class TestWriterApp(GoodAppTestCase):
             self.assertTrue(os.path.isdir('dir_or_file.d'))
             self.assertTrue(os.path.isfile('dir_or_file.d/index.html'))
             self.assertTrue(os.path.isfile('dir_or_file'))
+
+    @wpull.testing.async.async_test(timeout=DEFAULT_TIMEOUT)
+    def test_adjust_extension(self):
+        arg_parser = AppArgumentParser()
+        args = arg_parser.parse_args([
+            self.get_url('/mordor'),
+            self.get_url('/mordor?ring.asp'),
+            self.get_url('/mordor?ring.htm'),
+            self.get_url('/static/my_file.txt'),
+            self.get_url('/static/style.css'),
+            self.get_url('/static/style.css?hamster.exe'),
+            self.get_url('/static/mojibake.html'),
+            self.get_url('/static/mojibake.html?dolphin.png'),
+            '--adjust-extension',
+            '--no-host-directories',
+        ])
+
+        with cd_tempdir() as temp_dir:
+            builder = Builder(args, unit_test=True)
+            app = builder.build()
+            exit_code = yield From(app.run())
+
+            self.assertEqual(0, exit_code)
+
+            print(list(os.walk('.')))
+
+            self.assertTrue(os.path.isfile('mordor.html'))
+            self.assertTrue(os.path.isfile('mordor?ring.asp.html'))
+            self.assertTrue(os.path.isfile('mordor?ring.htm'))
+            self.assertTrue(os.path.isfile('static/my_file.txt'))
+            self.assertTrue(os.path.isfile('static/style.css'))
+            self.assertTrue(os.path.isfile('static/style.css?hamster.exe.css'))
+            self.assertTrue(os.path.isfile('static/mojibake.html'))
+            self.assertTrue(os.path.isfile('static/mojibake.html?dolphin.png.html'))
+
+    @wpull.testing.async.async_test(timeout=DEFAULT_TIMEOUT)
+    def test_content_disposition(self):
+        arg_parser = AppArgumentParser()
+        args = arg_parser.parse_args([
+            self.get_url('/content_disposition?filename=hello1.txt'),
+            self.get_url('/content_disposition?filename=hello2.txt;'),
+            self.get_url('/content_disposition?filename="hello3.txt"'),
+            self.get_url('/content_disposition?filename=\'hello4.txt\''),
+            '--content-disposition',
+            '--no-host-directories',
+        ])
+
+        with cd_tempdir() as temp_dir:
+            builder = Builder(args, unit_test=True)
+            app = builder.build()
+            exit_code = yield From(app.run())
+
+            self.assertEqual(0, exit_code)
+
+            print(list(os.walk('.')))
+
+            self.assertTrue(os.path.isfile('hello1.txt'))
+            self.assertTrue(os.path.isfile('hello2.txt'))
+            self.assertTrue(os.path.isfile('hello3.txt'))
+            self.assertTrue(os.path.isfile('hello4.txt'))
+
+    @wpull.testing.async.async_test(timeout=DEFAULT_TIMEOUT)
+    def test_trust_server_names(self):
+        arg_parser = AppArgumentParser()
+        args = arg_parser.parse_args([
+            self.get_url('/redirect'),
+            '--trust-server-names',
+            '--no-host-directories',
+            ])
+
+        with cd_tempdir() as temp_dir:
+            builder = Builder(args, unit_test=True)
+            app = builder.build()
+            exit_code = yield From(app.run())
+
+            self.assertEqual(0, exit_code)
+
+            print(list(os.walk('.')))
+
+            self.assertTrue(os.path.isfile('index.html'))
